@@ -7,7 +7,7 @@ import time
 import pylab
 
 #get the needed properties to build the grid:
-input_file = './craters_params.txt'
+input_file = './craters_params_init.txt'
 inputs = ModelParameterDictionary(input_file)
 nrows = inputs.read_int('nrows')
 ncols = inputs.read_int('ncols')
@@ -18,13 +18,13 @@ nt = inputs.read_int('number_of_craters_per_loop')
 loops = inputs.read_int('number_of_loops')
 
 mg = RasterModelGrid(nrows, ncols, dx)
-mg.set_inactive_boundaries(False, False, False, False)
+mg.set_looped_boundaries(True, True)
 
 #create the fields in the grid
 mg.create_node_array_zeros('planet_surface__elevation')
 z = mg.create_node_array_zeros() + leftmost_elev
 z += initial_slope*np.amax(mg.node_y) - initial_slope*mg.node_y
-mg['node'][ 'planet_surface__elevation'] = z + np.random.rand(len(z))/10000.
+mg['node'][ 'planet_surface__elevation'] = z #+ np.random.rand(len(z))/10000.
 
 # Display a message
 print( 'Running ...' )
@@ -43,25 +43,28 @@ az = np.empty(nt)
 mass_balance = np.empty(nt)
 for i in xrange(loops):
     for j in xrange(nt):
-        mg = craters_component.excavate_a_crater(mg)
-        x[j] = craters_component.impact_property_dict['x']
-        y[j] = craters_component.impact_property_dict['y']
-        r[j] = craters_component.impact_property_dict['r']
-        slope[j] = craters_component.impact_property_dict['surface_slope']
-        angle[j] = craters_component.impact_property_dict['normal_angle']
-        az[j] = craters_component.impact_property_dict['impact_az']
-        mass_balance[j] = craters_component.impact_property_dict['mass_balance']
+        if j == 1:
+            craters_component._ycoord = 0.45*(mg.get_grid_xdimension()-mg.dx)
+            craters_component._radius /= 10.
+        mg = craters_component.excavate_a_crater_furbish(mg)
+        x[j] = craters_component.impact_xy_location[0]
+        y[j] = craters_component.impact_xy_location[1]
+        r[j] = craters_component.crater_radius
+        slope[j] = craters_component.surface_slope_beneath_crater
+        angle[j] = craters_component.impact_angle_to_normal
+        az[j] = craters_component.impactor_travel_azimuth
+        mass_balance[j] = craters_component.mass_balance
         print 'Completed loop ', j
-    mystring = 'craterssave'+str(i*nt)
+    mystring = 'initial_topo'  #'craterssave'+str((i+1)*nt)
     np.save(mystring,mg['node']['planet_surface__elevation'])
     #Save the properties
-    np.save(('x_'+str(i*nt)),x)
-    np.save(('y_'+str(i*nt)),y)
-    np.save(('r_'+str(i*nt)),r)
-    np.save(('slope_'+str(i*nt)),slope)
-    np.save(('angle_'+str(i*nt)),angle)
-    np.save(('az_'+str(i*nt)),az)
-    np.save(('mass_balance_'+str(i*nt)),mass_balance)
+    np.save(('x_'+str((i+1)*nt)),x)
+    np.save(('y_'+str((i+1)*nt)),y)
+    np.save(('r_'+str((i+1)*nt)),r)
+    np.save(('slope_'+str((i+1)*nt)),slope)
+    np.save(('angle_'+str((i+1)*nt)),angle)
+    np.save(('az_'+str((i+1)*nt)),az)
+    np.save(('mass_balance_'+str((i+1)*nt)),mass_balance)
 
 #Finalize and plot
 elev = mg['node']['planet_surface__elevation']
@@ -78,3 +81,6 @@ print('Done.')
 print('Total run time = '+str(time.time()-start_time)+' seconds.')
 
 pylab.show()
+
+im=pylab.plot(elev_r[:,ncols//2])
+pylab.show()
\ No newline at end of file
