@@ -1,4 +1,9 @@
 #!/usr/bin/env python
+"""
+.. codeauthor:: Eric Hutton <huttone@colorado.edu>
+
+.. sectionauthor:: Eric Hutton <huttone@colorado.edu>
+"""
 
 from landlab.components.flexure.flexure import FlexureComponent
 from landlab.components.flexure.funcs import (get_flexure_parameter,
