@@ -65,12 +65,10 @@ def cell_count(shape):
     """
     assert(len(shape) == 2)
 
-    try:
-        assert(np.min(shape) > 2)
-    except AssertionError:
-        return 0
-    else:
+    if np.min(shape) > 2:
         return (shape[0] - 2) * (shape[1] - 2)
+    else:
+        return 0
 
 
 def active_cell_count(shape):
@@ -84,7 +82,7 @@ def active_cell_count(shape):
 def active_link_count(shape):
     """
     Number of active links in a structured grid with dimensions, *shape*.
-    S link is active if it connects to at least one active node.
+    A link is active if it connects to at least one active node.
 
     >>> link_count((3, 2))
     7
@@ -115,6 +113,16 @@ def link_count(shape):
     return shape[1] * (shape[0] - 1) + shape[0] * (shape[1] - 1)
 
 
+def vertical_link_count(shape):
+    assert(len(shape) == 2)
+    return (shape[0] - 1) * shape[1]
+
+
+def horizontal_link_count(shape):
+    assert(len(shape) == 2)
+    return shape[0] * (shape[1] - 1)
+
+
 def boundary_cell_count(shape):
     """
     Number of cells that are on the boundary of a structured grid with
@@ -144,7 +152,24 @@ def face_count(shape):
     >>> face_count((3, 4))
     7
     """
-    return (shape[0] - 1) * (shape[1] - 2) + (shape[0] - 2) * (shape[1] - 1)
+    assert(len(shape) == 2)
+    if np.min(shape) > 2:
+        return ((shape[0] - 1) * (shape[1] - 2) +
+                (shape[0] - 2) * (shape[1] - 1))
+    else:
+        return 0
+
+
+def active_face_count(shape):
+    """
+    Total number of active faces in a structured grid with dimensions,
+    *shape*. Each cell has four faces, and shared faces only count once.
+    An active face is one that has a corresponing active link.
+
+    >>> active_face_count((3, 4))
+    7
+    """
+    return face_count(shape)
 
 
 def top_index_iter(shape):
@@ -240,7 +265,12 @@ def boundary_nodes(shape):
     array([ 0,  1,  2,  3,  4,  7,  8,  9, 10, 11])
     """
     return np.fromiter(boundary_iter(shape), dtype=np.int)
-
+    
+def corners(shape):
+    """
+    An array of the indices of the grid corner nodes.
+    """
+    return np.array([0, shape[1]-1, shape[1]*(shape[0]-1), shape[1]*shape[0]-1])
 
 def bottom_edge_node_ids(shape):
     return np.fromiter(bottom_index_iter(shape), dtype=np.int)
@@ -280,9 +310,8 @@ def interior_nodes(shape):
 def node_coords(shape, *args):
     """
     Get x, y coordinates for nodes in a structured grid with dimensions,
-    *shape*. Use the optional argument *spacing* to give to give the spacing
-    in each dimension, and *origin* the start of the coordinates in each
-    dimension.
+    *shape*. Use the optional argument *spacing* to give the spacing in each
+    dimension, and *origin* the start of the coordinates in each dimension.
 
     >>> (cols, rows) = node_coords((3, 2))
     >>> rows
@@ -306,10 +335,8 @@ def node_coords(shape, *args):
 
     node_count = np.prod(shape)
 
-    row_y = np.arange(origin[0], shape[0] * spacing[0] + origin[0],
-                      spacing[0])
-    col_x = np.arange(origin[1], shape[1] * spacing[1] + origin[0],
-                      spacing[1])
+    row_y = np.arange(shape[0]) * spacing[0] + origin[0] 
+    col_x = np.arange(shape[1]) * spacing[1] + origin[1]
 
     (node_x, node_y) = np.meshgrid(col_x, row_y)
 
@@ -319,7 +346,7 @@ def node_coords(shape, *args):
     return (node_x, node_y)
 
 
-def active_cells(shape):
+def active_cell_index(shape):
     """
     Ordered indices of the active cells of a structured grid.
     """
@@ -329,22 +356,22 @@ def active_cells(shape):
 def active_cell_node(shape):
     """
     Indices of the nodes belonging to each cell. Since all cells are active,
-    this is the same as cell_node_index.
+    this is the same as node_index_at_cells.
 
-    >>> active_cell_node((4,3))
+    >>> node_index_at_cells((4,3))
     array([4, 7])
     """
-    return cell_node_index(shape)
+    return node_index_at_cells(shape)
 
 
-def node_active_cell(shape, boundary_node_index=BAD_INDEX_VALUE):
+def active_cell_index_at_nodes(shape, boundary_node_index=BAD_INDEX_VALUE):
     """
     Indices of the cells associated with the nodes of the structured grid.
     For nodes that don't have a cell (that is, boundary nodes) set indices
     to BAD_INDEX_VALUE. Use the *boundary_node_index* keyword to change
     the value of indices to boundary nodes.
 
-    >>> node_active_cell((3, 4), boundary_node_index=-1) # doctest: +NORMALIZE_WHITESPACE
+    >>> active_cell_index_at_nodes((3, 4), boundary_node_index=-1) # doctest: +NORMALIZE_WHITESPACE
     array([-1, -1, -1, -1,
            -1,  0,  1, -1,
            -1, -1, -1, -1])
@@ -359,11 +386,11 @@ def node_active_cell(shape, boundary_node_index=BAD_INDEX_VALUE):
     return node_ids
 
 
-def cell_node_index(shape):
+def node_index_at_cells(shape):
     """
     Indices of the nodes belonging to each cell.
 
-    >>> cell_node_index((4, 3))
+    >>> node_index_at_cells((4, 3))
     array([4, 7])
     """
     node_ids = np.arange(node_count(shape))
@@ -375,40 +402,36 @@ def cell_node_index(shape):
     return cell_node
 
 
-def node_link_index(shape):
+def node_index_at_link_ends(shape):
     node_ids = np.arange(np.prod(shape))
     node_ids.shape = shape
 
-    return (from_node_links(node_ids), to_node_links(node_ids))
+    return (node_index_at_link_head(node_ids),
+            node_index_at_link_tail(node_ids))
 
 
-def node_tolink_index(shape):
-    node_ids = np.arange(np.prod(shape))
-    node_ids.shape = shape
-
-    return to_node_links(node_ids)
+def inlink_index_at_node(shape):
+    return inlinks(shape, return_count=False)
 
 
-def node_fromlink_index(shape):
-    node_ids = np.arange(np.prod(shape))
-    node_ids.shape = shape
-
-    return from_node_links(node_ids)
+def outlink_index_at_node(shape):
+    return outlinks(shape, return_count=False)
 
 
-def to_node_links(node_ids):
+def node_index_at_link_tail(node_ids):
     vertical_links = node_ids[1:, :]
     horizontal_links = node_ids[:, 1:]
     return np.concatenate((vertical_links.flat, horizontal_links.flat))
 
 
-def from_node_links(node_ids):
+def node_index_at_link_head(node_ids):
     vertical_links = node_ids[:-1, :]
     horizontal_links = node_ids[:, :-1]
     return np.concatenate((vertical_links.flat, horizontal_links.flat))
 
 
-def link_faces(shape, actives=None, inactive_link_index=BAD_INDEX_VALUE):
+def face_index_at_links(shape, actives=None,
+                        inactive_link_index=BAD_INDEX_VALUE):
     """
     Returns an array that maps link ids to face ids. For inactive links,
     which do not have associated faces, set their ids to
@@ -417,7 +440,7 @@ def link_faces(shape, actives=None, inactive_link_index=BAD_INDEX_VALUE):
     that only the perimeter nodes are inactive.
 
 
-    >>> faces = link_faces((3, 4), inactive_link_index=-1)
+    >>> faces = face_index_at_links((3, 4), inactive_link_index=-1)
     >>> faces # doctest: +NORMALIZE_WHITESPACE
     array([-1,  0,  1, -1, -1,  2,  3,
            -1, -1, -1, -1,  4,  5,  6, -1, -1, -1])
@@ -458,7 +481,7 @@ def active_links(shape, node_status_array=None, link_nodes=None):
         node_status_array = node_status(shape)
 
     if link_nodes is None:
-        (link_from_node, link_to_node) = node_link_index(shape)
+        (link_from_node, link_to_node) = node_index_at_link_ends(shape)
     else:
         (link_from_node, link_to_node) = link_nodes
 
@@ -475,82 +498,343 @@ def active_links(shape, node_status_array=None, link_nodes=None):
     return active_links
 
 
-def setup_inlink_matrix(shape, tonodes=None, return_count=True):
-    """
-    >>> (link_ids, link_counts) = setup_inlink_matrix((3, 4))
-    >>> link_counts  # doctest: +NORMALIZE_WHITESPACE
-    array([0, 1, 1, 1,
-           1, 2, 2, 2,
-           1, 2, 2, 2])
-    >>> link_ids[0]  # doctest: +NORMALIZE_WHITESPACE
-    array([-1,  8,  9, 10,
-            0,  1,  2,  3,
-            4,  5,  6,  7])
-    >>> link_ids[1]  # doctest: +NORMALIZE_WHITESPACE
-    array([-1, -1, -1, -1,
-           -1, 11, 12, 13,
-           -1, 14, 15, 16])
-    """
-    num_nodes = node_count(shape)
+def active_face_index(shape):
+    return np.arange(active_face_count(shape))
 
-    #node_inlink_matrix = - np.ones((2, num_nodes), dtype=np.int)
-    node_inlink_matrix = np.empty((2, num_nodes), dtype=np.int)
-    node_inlink_matrix.fill(BAD_INDEX_VALUE)
 
-    if tonodes is None:
-        tonodes = node_tolink_index(shape)
+def inlinks(shape, tonodes=None, return_count=True):
+    links = np.vstack((south_links(shape), west_links(shape)))
+    links.shape = (2, node_count(shape))
+    return links
 
-    node_numinlink = np.bincount(tonodes, minlength=num_nodes)
-    counts = count_repeated_values(tonodes)
-    for (count, (tonodes, link_ids)) in enumerate(counts):
-        node_inlink_matrix[count][tonodes] = link_ids
 
-    node_inlink_matrix.sort(axis=0)
-    node_inlink_matrix[node_inlink_matrix == BAD_INDEX_VALUE] = -1
-    
-    if return_count:
-        return (node_inlink_matrix, node_numinlink)
+def outlinks(shape, tonodes=None, return_count=True):
+    links = np.vstack((north_links(shape), east_links(shape)))
+    links.shape = (2, node_count(shape))
+    return links
+
+
+def active_inlinks(shape, node_status=None, return_count=True):
+    links = np.vstack((active_south_links(shape, node_status),
+                       active_west_links(shape, node_status)))
+    links.shape = (2, node_count(shape))
+    return links
+
+
+def active_outlinks(shape, node_status=None, return_count=True):
+    links = np.vstack((active_north_links(shape, node_status),
+                       active_east_links(shape, node_status)))
+    links.shape = (2, node_count(shape))
+    return links
+
+
+def vertical_link_ids(shape):
+    link_ids = np.arange(0, vertical_link_count(shape), dtype=np.int)
+    link_ids.shape = (shape[0] - 1, shape[1])
+    return link_ids
+
+
+def horizontal_link_ids(shape):
+    link_id_offset = vertical_link_count(shape)
+    link_ids = np.arange(link_id_offset,
+                         link_id_offset + horizontal_link_count(shape),
+                         dtype=np.int)
+    link_ids.shape = (shape[0], shape[1] - 1)
+    return link_ids
+
+
+def vertical_active_link_count(shape, node_status=None):
+    if node_status is not None:
+        is_inactive = (node_status == 0)
+        is_inactive.shape = shape
+
+        inactive_outlinks = is_inactive[:-1, 1:-1]
+        inactive_inlinks = is_inactive[1:, 1:-1]
+        inactive_links = inactive_outlinks | inactive_inlinks
+
+        return (shape[0] - 1) * (shape[1] - 2) - np.sum(inactive_links.flat)
     else:
-        return node_inlink_matrix
+        return (shape[0] - 1) * (shape[1] - 2)
+
+
+def horizontal_active_link_count(shape, node_status=None):
+    if node_status is not None:
+        is_inactive = (node_status == 0)
+        is_inactive.shape = shape
+
+        inactive_outlinks = is_inactive[1:-1, :-1]
+        inactive_inlinks = is_inactive[1:-1, 1:]
+        inactive_links = inactive_outlinks | inactive_inlinks
+
+        return (shape[0] - 2) * (shape[1] - 1) - np.sum(inactive_links.flat)
+    else:
+        return (shape[0] - 2) * (shape[1] - 1)
+
+
+def vertical_inactive_link_mask(shape, node_status):
+    is_inactive = (node_status == 0)
+    is_inactive.shape = shape
+
+    inactive_outlinks = is_inactive[:-1, 1:-1]
+    inactive_inlinks = is_inactive[1:, 1:-1]
+    return inactive_outlinks | inactive_inlinks
+
+
+def horizontal_inactive_link_mask(shape, node_status):
+    is_inactive = (node_status == 0)
+    is_inactive.shape = shape
+
+    inactive_outlinks = is_inactive[1:-1, :-1]
+    inactive_inlinks = is_inactive[1:-1, 1:]
+    return inactive_outlinks | inactive_inlinks
+
+
+#def vertical_active_link_ids(shape):
+#    link_ids = np.arange(vertical_active_link_count(shape), dtype=np.int)
+#    link_ids.shape = (shape[0] - 1, shape[1] - 2)
+#    return link_ids
+
+
+def vertical_active_link_ids(shape, node_status=None):
+    if node_status is None:
+        link_ids = np.arange(vertical_active_link_count(shape), dtype=np.int)
+        #link_ids.shape = (shape[0] - 1, shape[1] - 2)
+        #return link_ids
+    else:
+        inactive_links = vertical_inactive_link_mask(shape, node_status)
+        inactive_links.shape = (inactive_links.size, )
+        active_link_count = inactive_links.size - np.sum(inactive_links)
+
+        link_ids = np.empty(inactive_links.size)
+        link_ids[inactive_links] = - 1
+        link_ids[~ inactive_links] = np.arange(active_link_count, dtype=int)
+
+    link_ids.shape = (shape[0] - 1, shape[1] - 2)
+    return link_ids
+
+
+def horizontal_active_link_ids(shape, node_status=None):
+    if node_status is None:
+        link_id_offset = vertical_active_link_count(shape)
+        link_ids = np.arange(
+            link_id_offset,
+            link_id_offset + horizontal_active_link_count(shape),
+            dtype=np.int)
+    else:
+        link_id_offset = vertical_active_link_count(shape,
+                                                    node_status=node_status)
+        inactive_links = horizontal_inactive_link_mask(shape, node_status)
+        inactive_links.shape = (inactive_links.size, )
+        active_link_count = inactive_links.size - np.sum(inactive_links)
+
+        link_ids = np.empty(inactive_links.size)
+        link_ids[inactive_links] = - 1
+        link_ids[~ inactive_links] = np.arange(
+            link_id_offset, link_id_offset + active_link_count,
+            dtype=np.int)
+    link_ids.shape = (shape[0] - 2, shape[1] - 1)
+    return link_ids
+
+
+def west_links(shape):
+    """
+    >>> west_links((3, 4))
+    array([[-1,  8,  9, 10],
+           [-1, 11, 12, 13],
+           [-1, 14, 15, 16]])
+    """
+    link_ids = horizontal_link_ids(shape)
+    link_ids.shape = (shape[0], shape[1] - 1)
+    return np.hstack((- np.ones((shape[0], 1), dtype=np.int), link_ids))
+
+
+def north_links(shape):
+    """
+    >>> north_links((3, 4))
+    array([[ 0,  1,  2,  3],
+           [ 4,  5,  6,  7],
+           [-1, -1, -1, -1]])
+    """
+    link_ids = vertical_link_ids(shape)
+    link_ids.shape = (shape[0] - 1, shape[1])
+    return np.vstack((link_ids, - np.ones((1, shape[1]), dtype=np.int)))
+
+
+def south_links(shape):
+    """
+    >>> south_links((3, 4))
+    array([[-1, -1, -1, -1],
+           [ 0,  1,  2,  3],
+           [ 4,  5,  6,  7]])
+    """
+    link_ids = vertical_link_ids(shape)
+    link_ids.shape = (shape[0] - 1, shape[1])
+    return np.vstack((- np.ones((1, shape[1]), dtype=np.int), link_ids))
+
+def east_links(shape):
+    """
+    >>> east_links((3, 4))
+    array([[ 8,  9, 10, -1],
+           [11, 12, 13, -1],
+           [14, 15, 16, -1]])
+    """
+    link_ids = horizontal_link_ids(shape)
+    link_ids.shape = (shape[0], shape[1] - 1)
+    return np.hstack((link_ids, - np.ones((shape[0], 1), dtype=int)))
+
+
+def active_north_links(shape, node_status=None):
+    """
+    >>> active_north_links((3, 4))
+    array([[-1,  0,  1, -1],
+           [-1,  2,  3, -1],
+           [-1, -1, -1, -1]])
+    """
+    active_north_links = np.empty(shape, dtype=int)
+    try:
+        links = vertical_active_link_ids(shape, node_status=node_status)
+    except ValueError:
+        pass
+    links.shape = (shape[0] - 1, shape[1] - 2)
+    active_north_links[:-1, 1:-1] = links
+    active_north_links[:, (0, -1)] = -1
+    active_north_links[-1, :] = -1
+
+    return active_north_links
+
+
+def active_south_links(shape, node_status=None):
+    """
+    >>> active_south_links((3, 4))
+    array([[-1, -1, -1, -1],
+           [-1,  0,  1, -1],
+           [-1,  2,  3, -1]])
+    """
+    active_south_links = np.empty(shape, dtype=int)
+    links = vertical_active_link_ids(shape, node_status=node_status)
+    links.shape = (shape[0] - 1, shape[1] - 2)
+    active_south_links[1:, 1:-1] = links
+    active_south_links[:, (0, -1)] = -1
+    active_south_links[0, :] = -1
+
+    return active_south_links
+
+
+def active_west_links(shape, node_status=None):
+    """
+    >>> active_west_links((3, 4))
+    array([[-1, -1, -1, -1],
+           [-1,  4,  5,  6],
+           [-1, -1, -1, -1]])
+    """
+    active_west_links = np.empty(shape, dtype=int)
+    try:
+        active_west_links[1:-1, 1:] = horizontal_active_link_ids(
+            shape, node_status=node_status)
+    except ValueError:
+        pass
+    active_west_links[(0, -1), :] = -1
+    active_west_links[:, 0] = -1
+
+    return active_west_links
+
+
+def active_east_links(shape, node_status=None):
+    """
+    >>> active_east_links((3, 4))
+    array([[-1, -1, -1, -1],
+           [ 4,  5,  6, -1],
+           [-1, -1, -1, -1]])
+    """
+    active_east_links = np.empty(shape, dtype=int)
+    active_east_links.fill(-999)
+    try:
+        active_east_links[1:-1, :-1] = horizontal_active_link_ids(
+            shape, node_status=node_status)
+    except ValueError:
+        pass
+    active_east_links[(0, -1), :] = -1
+    active_east_links[:, -1] = -1
+
+    return active_east_links
+
+
+def outlink_count_per_node(shape):
+    link_count = np.empty(shape, dtype=np.int)
+    link_count[:-1, :-1] = 2
+    link_count[-1, :-1] = 1
+    link_count[:-1, -1] = 1
+    link_count[-1, -1] = 0
+    return np.ravel(link_count)
+
+
+def inlink_count_per_node(shape):
+    link_count = np.empty(shape, dtype=np.int)
+    link_count[1:, 1:] = 2
+    link_count[0, 1:] = 1
+    link_count[1:, 0] = 1
+    link_count[0, 0] = 0
+    return np.ravel(link_count)
+
+
+def active_outlink_count_per_node(shape):
+    link_count = np.empty(shape, dtype=np.int)
+    link_count[1:-1, 1:-1] = 2
+    link_count[0, :] = 1
+    link_count[-1, :] = 0
+    link_count[:, 0] = 1
+    link_count[:, -1] = 0
+
+    link_count[0, 0] = 0
+    link_count[-1, 0] = 0
+
+    return np.ravel(link_count)
+
+
+def active_inlink_count_per_node(shape):
+    link_count = np.empty(shape, dtype=np.int)
+    link_count[1:-1, 1:-1] = 2
+    link_count[0, :] = 0
+    link_count[-1, :] = 1
+    link_count[:, 0] = 0
+    link_count[:, -1] = 1
+
+    link_count[0, -1] = 0
+    link_count[-1, -1] = 0
+
+    return np.ravel(link_count)
 
 
 def setup_outlink_matrix(shape, fromnodes=None, return_count=True):
-    """
-    >>> (link_ids, link_counts) = setup_outlink_matrix((3, 4)) 
-    >>> link_counts # doctest: +NORMALIZE_WHITESPACE
-    array([2, 2, 2, 1,
-           2, 2, 2, 1,
-           1, 1, 1, 0])
-    >>> link_ids[0] # doctest: +NORMALIZE_WHITESPACE
-    array([ 0,  1,  2,  3,
-            4,  5,  6,  7,
-           14, 15, 16, -1])
-    >>> link_ids[1] # doctest: +NORMALIZE_WHITESPACE
-    array([ 8,  9, 10, -1,
-           11, 12, 13, -1,
-           -1, -1, -1, -1])
-    """
-    num_nodes = node_count(shape)
-
-    #node_outlink_matrix = - np.ones((2, num_nodes), dtype=np.int)
-    node_outlink_matrix = np.empty((2, num_nodes), dtype=np.int)
-    node_outlink_matrix.fill(BAD_INDEX_VALUE)
-
-    if fromnodes is None:
-        fromnodes = node_fromlink_index(shape)
-
-    node_numoutlink = np.bincount(fromnodes, minlength=num_nodes)
-    counts = count_repeated_values(fromnodes)
-    for (count, (fromnodes, link_ids)) in enumerate(counts):
-        node_outlink_matrix[count][fromnodes] = link_ids
-
-    node_outlink_matrix.sort(axis=0)
-    node_outlink_matrix[node_outlink_matrix == BAD_INDEX_VALUE] = -1
-    
+    links = outlinks(shape)
     if return_count:
-        return (node_outlink_matrix, node_numoutlink)
+        return (links, outlink_count_per_node(shape))
     else:
-        return node_outlink_matrix
+        return links
+
+
+def setup_inlink_matrix(shape, tonodes=None, return_count=True):
+    links = inlinks(shape)
+    if return_count:
+        return (links, inlink_count_per_node(shape))
+    else:
+        return links
+
+
+def setup_active_outlink_matrix(shape, node_status=None, return_count=True):
+    links = active_outlinks(shape, node_status=node_status)
+    if return_count:
+        return (links, active_outlink_count_per_node(shape))
+    else:
+        return links
+
+
+def setup_active_inlink_matrix(shape, node_status=None, return_count=True):
+    links = active_inlinks(shape, node_status=node_status)
+    if return_count:
+        return (links, active_inlink_count_per_node(shape))
+    else:
+        return links
 
 
 def node_index_with_halo(shape, halo_indices=BAD_INDEX_VALUE):
